==> zeroconf-0.1.tgz: zeroconf-0.1/zeroconf.c <==
/* ==================================================================
 * ZeroConf Extension for PHP
 * Copyright (c) 2004 Gabriel Ricard
 * All rights reserved.
 * ==================================================================
 * This source code is made available free and without charge subject
 * to the terms of the PHP Group License as detailed in bundled LICENSE file.
 * ==================================================================
 * Gabriel Ricard <gabe@php.net>
 * ==================================================================
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_zeroconf.h"
#include <howl.h>

/* If you declare any globals in php_zeroconf.h uncomment this:
ZEND_DECLARE_MODULE_GLOBALS(zeroconf)
*/

/* True global resources - no need for thread safety here */
static int le_zeroconf;

typedef struct _zeroconf_session {
	sw_discovery *session;
	zval *browse_callback;
	zval *publish_callback;
} zeroconf_session;


/* {{{ zeroconf_functions[]
 */
function_entry zeroconf_functions[] = {
	PHP_FE(zeroconf_session_init,	NULL)
	PHP_FE(zeroconf_browse_service,	NULL)
	PHP_FE(zeroconf_browse_process,	NULL)
	PHP_FE(zeroconf_session_destroy,	NULL)
	{NULL, NULL, NULL}
};
/* }}} */

/* {{{ zeroconf_module_entry
 */
zend_module_entry zeroconf_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"zeroconf",
	zeroconf_functions,
	PHP_MINIT(zeroconf),
	PHP_MSHUTDOWN(zeroconf),
	PHP_RINIT(zeroconf),		/* Replace with NULL if there's nothing to do at request start */
	PHP_RSHUTDOWN(zeroconf),	/* Replace with NULL if there's nothing to do at request end */
	PHP_MINFO(zeroconf),
#if ZEND_MODULE_API_NO >= 20010901
	NO_VERSION_YET,
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */


#ifdef COMPILE_DL_ZEROCONF
ZEND_GET_MODULE(zeroconf)
#endif

/* Remove comments and fill if you need to have entries in php.ini
PHP_INI_BEGIN()
    STD_PHP_INI_ENTRY("zeroconf.global_value",      "42", PHP_INI_ALL, OnUpdateInt, global_value, zend_zeroconf_globals, zeroconf_globals)
    STD_PHP_INI_ENTRY("zeroconf.global_string", "foobar", PHP_INI_ALL, OnUpdateString, global_string, zend_zeroconf_globals, zeroconf_globals)
PHP_INI_END()
*/

static void _zeroconf_session_dtor(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	if( rsrc->ptr ) {
		zeroconf_session *zc_session = rsrc->ptr;

		sw_discovery_fina(zc_session->session);

		if (zc_session->browse_callback != NULL) {
			zval_ptr_dtor(&zc_session->browse_callback);
		}

		if (zc_session->publish_callback != NULL) {
			zval_ptr_dtor(&zc_session->publish_callback);
		}

		efree(zc_session->session);
		efree(zc_session);
		rsrc->ptr = NULL;
	}
}

static sw_result HOWL_API 
zc_resolver_handler(
					sw_discovery_resolve_handler  handler,
					sw_discovery                  discovery,
					sw_discovery_resolve_id       id,
					sw_const_string               name,
					sw_const_string               type,
					sw_const_string               domain,
					sw_ipv4_address               address,
					sw_port                       port,
					sw_const_string               text_record_string,
					sw_octets                     text_record,
					sw_ulong                      text_record_len,
					sw_opaque                     extra
					)
{
	sw_text_record_string_iterator  text_str_iterator;
	char                            name_buf[16];
	sw_char                         key[SW_TEXT_RECORD_MAX_LEN];
	sw_char                         s_val[SW_TEXT_RECORD_MAX_LEN];

	int cb_res;
	zval *retval, *callback;
	zval *arg_ip, *arg_port, *arg_text_records, **args[3];
	zeroconf_session *zc_session = (zeroconf_session *)extra;

	TSRMLS_FETCH();

	if( extra != NULL ) {
		callback = zc_session->browse_callback;
	}

	sw_discovery_stop_resolve(discovery, id);
	sw_ipv4_address_name(address, name_buf, 16);

	/* assemble ip, port and text records to pass to callback */
	args[0] = &arg_ip;
	args[1] = &arg_port;
	args[2] = &arg_text_records;

	MAKE_STD_ZVAL(arg_ip);
	ZVAL_STRING(arg_ip, name_buf, 1);

	MAKE_STD_ZVAL(arg_port);
	ZVAL_LONG(arg_port, port);

	MAKE_STD_ZVAL(arg_text_records);
	array_init(arg_text_records);

	if (text_record_len > 0 && text_record && *text_record != '\0') {
		SW_TRY(sw_text_record_string_iterator_init(&text_str_iterator, text_record_string));

		while (sw_text_record_string_iterator_next(text_str_iterator, key, s_val) == SW_OKAY)
		{
			add_assoc_string(arg_text_records, key, s_val, 1);
		}

		SW_TRY(sw_text_record_string_iterator_fina(text_str_iterator));
	}

	cb_res = call_user_function_ex(EG(function_table), NULL, callback, &retval, 3, args, 0, NULL TSRMLS_CC);

	if (cb_res == SUCCESS) {
		zval_ptr_dtor(&retval);
		zval_ptr_dtor(&arg_ip);
		zval_ptr_dtor(&arg_port);
		zval_ptr_dtor(&arg_text_records);
	} else {
		zval_ptr_dtor(&retval);
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "An error occurred while invoking the browse callback");
		return;
	}

	return SW_OKAY;
}


static sw_result HOWL_API
zc_browser_handler(	
				   sw_discovery_browse_handler  handler,
				   sw_discovery                 discovery,
				   sw_discovery_browse_id       id,
				   sw_discovery_browse_status   status,
				   sw_const_string              name,
				   sw_const_string              type,
				   sw_const_string              domain,
				   sw_opaque                    extra
				   )
{
	sw_discovery_resolve_id rid;

	switch (status) {
		case SW_DISCOVERY_BROWSE_ADD_SERVICE:
			if (sw_discovery_resolve(discovery, name, type, domain, NULL, zc_resolver_handler, extra, &rid) != SW_OKAY)
			{
				/*DISABLED fprintf(stderr, "resolve failed\n");*/
			}
		break;

		case SW_DISCOVERY_BROWSE_REMOVE_SERVICE:
			/*DISABLED fprintf(stderr, "remove service: %s %s %s\n", name, type, domain);*/
		break;

		case SW_DISCOVERY_BROWSE_INVALID:
		case SW_DISCOVERY_BROWSE_RELEASE:
		case SW_DISCOVERY_BROWSE_ADD_DOMAIN:
		case SW_DISCOVERY_BROWSE_ADD_DEFAULT_DOMAIN:
		case SW_DISCOVERY_BROWSE_REMOVE_DOMAIN:
		case SW_DISCOVERY_BROWSE_RESOLVED:
			/*DISABLED fprintf(stderr, "Some other action\n");*/
		break;
	}

	return SW_OKAY;
}


/* Uncomment this function if you have INI entries
static void php_zeroconf_init_globals(zend_zeroconf_globals *zeroconf_globals)
{
	zeroconf_globals->global_value = 0;
	zeroconf_globals->global_string = NULL;
}
*/

PHP_MINIT_FUNCTION(zeroconf)
{
	le_zeroconf = zend_register_list_destructors_ex(_zeroconf_session_dtor, NULL,
												  "ZeroConf Session",
												  module_number);

	/* If you have INI entries, uncomment these lines 
	ZEND_INIT_MODULE_GLOBALS(zeroconf, php_zeroconf_init_globals, NULL);
	REGISTER_INI_ENTRIES();
	*/

	return SUCCESS;
}

PHP_MSHUTDOWN_FUNCTION(zeroconf)
{
	/* uncomment this line if you have INI entries
	UNREGISTER_INI_ENTRIES();
	*/
	return SUCCESS;
}

PHP_RINIT_FUNCTION(zeroconf)
{
	return SUCCESS;
}

PHP_RSHUTDOWN_FUNCTION(zeroconf)
{
	return SUCCESS;
}

PHP_MINFO_FUNCTION(zeroconf)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "zeroconf support", "enabled");
	php_info_print_table_end();

	/* Remove comments if you have entries in php.ini
	DISPLAY_INI_ENTRIES();
	*/
}


/* {{{ proto resource zeroconf_session_init(void)
   Opens a ZeroConf session. */
PHP_FUNCTION(zeroconf_session_init)
{
	sw_result res;
	zeroconf_session *session_rsrc;

	if (ZEND_NUM_ARGS()) {
		WRONG_PARAM_COUNT;
	}

	session_rsrc = (zeroconf_session *) emalloc(sizeof(zeroconf_session));
	session_rsrc->session = (sw_discovery *) emalloc(sizeof(sw_discovery));
	session_rsrc->browse_callback = NULL;
	session_rsrc->publish_callback = NULL;

	if ((res = sw_discovery_init(session_rsrc->session)) != SW_OKAY) {
		RETURN_FALSE;
	}
	else {
		ZEND_REGISTER_RESOURCE(return_value, session_rsrc, le_zeroconf);
	}
}
/* }}} */

/* {{{ proto int zeroconf_browse_service(resource session, string service, string callback_function)
   Set service type to browse for. */
PHP_FUNCTION(zeroconf_browse_service)
{
	zval *session_rsrc, *callback, *cb_copy;
	zeroconf_session *zc_session;
	sw_discovery_browse_id id;
	sw_result res;
	char *svc_name, *callback_name;
	int svc_name_len;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rsz",
							  &session_rsrc,
							  &svc_name, &svc_name_len,
							  &callback
							  ) == FAILURE) {
		return;
	}

	if (!zend_is_callable(callback, 0, &callback_name)) {
		php_error_docref1(NULL TSRMLS_CC, callback_name, E_WARNING,
						  "%s is not a valid callback function", callback_name);
		efree(callback_name);
		RETURN_FALSE;
	}

	efree(callback_name);

	ZEND_FETCH_RESOURCE(zc_session, zeroconf_session *, &session_rsrc, -1, "ZeroConf Session", le_zeroconf);

	ALLOC_ZVAL(cb_copy);
	ZVAL_ADDREF(callback);
	COPY_PZVAL_TO_ZVAL(*cb_copy, callback);

	if (zc_session->browse_callback != NULL) {
		zval_ptr_dtor(&zc_session->browse_callback);
	}

	zc_session->browse_callback = cb_copy;

	if (sw_discovery_browse(*(zc_session->session), svc_name, NULL, NULL, zc_browser_handler, zc_session, &id) != SW_OKAY) {
		RETURN_FALSE;
	}

	RETURN_TRUE;
}
/* }}} */

/* {{{ proto void zeroconf_browse_process(resource session, int timeout_msec)
   Process ZeroConf browsing for specified time interval.  */
PHP_FUNCTION(zeroconf_browse_process)
{
	zval *session_rsrc;
	sw_result res;
	zeroconf_session *zc_session;
	sw_salt salt;
	sw_ulong msecs;
	double timeout_secs;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rd",
							  &session_rsrc, &timeout_secs) == FAILURE) {
		return;
	}

	ZEND_FETCH_RESOURCE(zc_session, sw_discovery *, &session_rsrc, -1, "ZeroConf Session", le_zeroconf);

	if (sw_discovery_salt(*(zc_session->session), &salt) != SW_OKAY) {
		fprintf(stderr, "salt failed\n");
		RETURN_FALSE;
	}

	msecs = (sw_ulong)timeout_secs * 1000;

	sw_salt_step(salt, &msecs);

	RETURN_TRUE;
}
/* }}} */

/* {{{ proto void zeroconf_session_destroy(resource session)
   End a ZeroConf session. */
PHP_FUNCTION(zeroconf_session_destroy)
{
	zval *session_rsrc;
	sw_result res;
	zeroconf_session *zc_session;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &session_rsrc) == FAILURE) {
		return;
	}

	ZEND_FETCH_RESOURCE(zc_session, sw_discovery *, &session_rsrc, -1, "ZeroConf Session", le_zeroconf);

	zend_list_delete(Z_RESVAL_P(session_rsrc));
}
/* }}} */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */

==> zeroconf-0.1.1.tgz: zeroconf-0.1.1/zeroconf.c <==
/* ==================================================================
 * ZeroConf Extension for PHP
 * Copyright (c) 2004 Gabriel Ricard
 * All rights reserved.
 * ==================================================================
 * This source code is made available free and without charge subject
 * to the terms of the PHP Group License as detailed in bundled LICENSE file.
 * ==================================================================
 * Gabriel Ricard <gabe@php.net>
 * ==================================================================
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_zeroconf.h"
#include <howl.h>

/* If you declare any globals in php_zeroconf.h uncomment this:
ZEND_DECLARE_MODULE_GLOBALS(zeroconf)
*/

/* True global resources - no need for thread safety here */
static int le_zeroconf;

typedef struct _zeroconf_session {
	sw_discovery *session;
	zval *browse_callback;
	zval *publish_callback;
} zeroconf_session;


/* {{{ zeroconf_functions[]
 */
function_entry zeroconf_functions[] = {
	PHP_FE(zeroconf_session_init,	NULL)
	PHP_FE(zeroconf_browse_service,	NULL)
	PHP_FE(zeroconf_browse_process,	NULL)
	PHP_FE(zeroconf_session_destroy,	NULL)
	{NULL, NULL, NULL}
};
/* }}} */

/* {{{ zeroconf_module_entry
 */
zend_module_entry zeroconf_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"zeroconf",
	zeroconf_functions,
	PHP_MINIT(zeroconf),
	PHP_MSHUTDOWN(zeroconf),
	PHP_RINIT(zeroconf),		/* Replace with NULL if there's nothing to do at request start */
	PHP_RSHUTDOWN(zeroconf),	/* Replace with NULL if there's nothing to do at request end */
	PHP_MINFO(zeroconf),
#if ZEND_MODULE_API_NO >= 20010901
	NO_VERSION_YET,
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */


#ifdef COMPILE_DL_ZEROCONF
ZEND_GET_MODULE(zeroconf)
#endif

/* Remove comments and fill if you need to have entries in php.ini
PHP_INI_BEGIN()
    STD_PHP_INI_ENTRY("zeroconf.global_value",      "42", PHP_INI_ALL, OnUpdateInt, global_value, zend_zeroconf_globals, zeroconf_globals)
    STD_PHP_INI_ENTRY("zeroconf.global_string", "foobar", PHP_INI_ALL, OnUpdateString, global_string, zend_zeroconf_globals, zeroconf_globals)
PHP_INI_END()
*/

static void _zeroconf_session_dtor(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	if( rsrc->ptr ) {
		zeroconf_session *zc_session = rsrc->ptr;
		sw_discovery *session = zc_session->session;

		sw_discovery_fina(*session);

		if (zc_session->browse_callback != NULL) {
			zval_ptr_dtor(&zc_session->browse_callback);
		}

		if (zc_session->publish_callback != NULL) {
			zval_ptr_dtor(&zc_session->publish_callback);
		}

		efree(zc_session->session);
		efree(zc_session);
		rsrc->ptr = NULL;
	}
}

static sw_result HOWL_API 
zc_resolver_handler(
					sw_discovery_resolve_handler  handler,
					sw_discovery                  discovery,
					sw_discovery_resolve_id       id,
					sw_const_string               name,
					sw_const_string               type,
					sw_const_string               domain,
					sw_ipv4_address               address,
					sw_port                       port,
					sw_const_string               text_record_string,
					sw_octets                     text_record,
					sw_ulong                      text_record_len,
					sw_opaque                     extra
					)
{
	sw_text_record_string_iterator  text_str_iterator;
	char                            name_buf[16];
	sw_char                         key[SW_TEXT_RECORD_MAX_LEN];
	sw_char                         s_val[SW_TEXT_RECORD_MAX_LEN];

	int cb_res;
	zval *retval, *callback, **args[6];
	zval *arg_ip, *arg_port, *arg_text_records;
	zval *arg_domain, *arg_type, *arg_name;
	zeroconf_session *zc_session = (zeroconf_session *)extra;

	TSRMLS_FETCH();

	if( extra != NULL ) {
		callback = zc_session->browse_callback;
	}

	sw_discovery_stop_resolve(discovery, id);
	sw_ipv4_address_name(address, name_buf, 16);

	/* assemble ip, port and text records to pass to callback */
	args[0] = &arg_ip;
	args[1] = &arg_port;
	args[2] = &arg_type;
	args[3] = &arg_name;
	args[4] = &arg_domain;
	args[5] = &arg_text_records;

	MAKE_STD_ZVAL(arg_ip);
	ZVAL_STRING(arg_ip, name_buf, 1);

	MAKE_STD_ZVAL(arg_port);
	ZVAL_LONG(arg_port, port);

	MAKE_STD_ZVAL(arg_type);
	ZVAL_STRING(arg_type, (char *)type, 1);

	MAKE_STD_ZVAL(arg_name);
	ZVAL_STRING(arg_name, (char *)name, 1);

	MAKE_STD_ZVAL(arg_domain);
	ZVAL_STRING(arg_domain, (char *)domain, 1);

	MAKE_STD_ZVAL(arg_text_records);
	array_init(arg_text_records);

	if (text_record_len > 0 && text_record && *text_record != '\0') {
		SW_TRY(sw_text_record_string_iterator_init(&text_str_iterator, text_record_string));

		while (sw_text_record_string_iterator_next(text_str_iterator, key, s_val) == SW_OKAY)
		{
			add_assoc_string(arg_text_records, key, s_val, 1);
		}

		SW_TRY(sw_text_record_string_iterator_fina(text_str_iterator));
	}

	cb_res = call_user_function_ex(EG(function_table), NULL, callback, &retval, 6, args, 0, NULL TSRMLS_CC);

	if (cb_res == SUCCESS) {
		zval_ptr_dtor(&retval);
		zval_ptr_dtor(&arg_ip);
		zval_ptr_dtor(&arg_port);
		zval_ptr_dtor(&arg_type);
		zval_ptr_dtor(&arg_name);
		zval_ptr_dtor(&arg_domain);
		zval_ptr_dtor(&arg_text_records);
	} else {
		zval_ptr_dtor(&retval);
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "An error occurred while invoking the browse callback");

		return 1;
	}

	return SW_OKAY;
}


static sw_result HOWL_API
zc_browser_handler(	
				   sw_discovery_browse_handler  handler,
				   sw_discovery                 discovery,
				   sw_discovery_browse_id       id,
				   sw_discovery_browse_status   status,
				   sw_const_string              name,
				   sw_const_string              type,
				   sw_const_string              domain,
				   sw_opaque                    extra
				   )
{
	sw_discovery_resolve_id rid;

	switch (status) {
		case SW_DISCOVERY_BROWSE_ADD_SERVICE:
			if (sw_discovery_resolve(discovery, name, type, domain, NULL, zc_resolver_handler, extra, &rid) != SW_OKAY)
			{
				/*DISABLED fprintf(stderr, "resolve failed\n");*/
			}
		break;

		case SW_DISCOVERY_BROWSE_REMOVE_SERVICE:
			/*DISABLED fprintf(stderr, "remove service: %s %s %s\n", name, type, domain);*/
		break;

		case SW_DISCOVERY_BROWSE_INVALID:
		case SW_DISCOVERY_BROWSE_RELEASE:
		case SW_DISCOVERY_BROWSE_ADD_DOMAIN:
		case SW_DISCOVERY_BROWSE_ADD_DEFAULT_DOMAIN:
		case SW_DISCOVERY_BROWSE_REMOVE_DOMAIN:
		case SW_DISCOVERY_BROWSE_RESOLVED:
			/*DISABLED fprintf(stderr, "Some other action\n");*/
		break;
	}

	return SW_OKAY;
}


/* Uncomment this function if you have INI entries
static void php_zeroconf_init_globals(zend_zeroconf_globals *zeroconf_globals)
{
	zeroconf_globals->global_value = 0;
	zeroconf_globals->global_string = NULL;
}
*/

PHP_MINIT_FUNCTION(zeroconf)
{
	le_zeroconf = zend_register_list_destructors_ex(_zeroconf_session_dtor, NULL,
												  "ZeroConf Session",
												  module_number);

	/* If you have INI entries, uncomment these lines 
	ZEND_INIT_MODULE_GLOBALS(zeroconf, php_zeroconf_init_globals, NULL);
	REGISTER_INI_ENTRIES();
	*/

	return SUCCESS;
}

PHP_MSHUTDOWN_FUNCTION(zeroconf)
{
	/* uncomment this line if you have INI entries
	UNREGISTER_INI_ENTRIES();
	*/
	return SUCCESS;
}

PHP_RINIT_FUNCTION(zeroconf)
{
	return SUCCESS;
}

PHP_RSHUTDOWN_FUNCTION(zeroconf)
{
	return SUCCESS;
}

PHP_MINFO_FUNCTION(zeroconf)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "zeroconf support", "enabled");
	php_info_print_table_end();

	/* Remove comments if you have entries in php.ini
	DISPLAY_INI_ENTRIES();
	*/
}


/* {{{ proto resource zeroconf_session_init(string callback_function)
   Opens a ZeroConf session. */
PHP_FUNCTION(zeroconf_session_init)
{
	zval *callback, *cb_copy;
	sw_result res;
	zeroconf_session *session_rsrc;
	char *callback_name;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &callback ) == FAILURE) {
		return;
	}

	if (!zend_is_callable(callback, 0, &callback_name)) {
		php_error_docref1(NULL TSRMLS_CC, callback_name, E_WARNING,
						  "%s is not a valid callback function", callback_name);
		efree(callback_name);
		RETURN_FALSE;
	}

	efree(callback_name);

	ALLOC_ZVAL(cb_copy);
	ZVAL_ADDREF(callback);
	COPY_PZVAL_TO_ZVAL(*cb_copy, callback);

	session_rsrc = (zeroconf_session *) emalloc(sizeof(zeroconf_session));
	session_rsrc->session = (sw_discovery *) emalloc(sizeof(sw_discovery));
	session_rsrc->browse_callback = cb_copy;
	session_rsrc->publish_callback = NULL;

	if ((res = sw_discovery_init(session_rsrc->session)) != SW_OKAY) {
		RETURN_FALSE;
	}
	else {
		ZEND_REGISTER_RESOURCE(return_value, session_rsrc, le_zeroconf);
	}
}
/* }}} */

/* {{{ proto int zeroconf_browse_service(resource session, string service)
   Set service type to browse for. */
PHP_FUNCTION(zeroconf_browse_service)
{
	zval *session_rsrc;
	zeroconf_session *zc_session;
	sw_discovery_browse_id id;
	char *svc_name;
	int svc_name_len;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs",
							  &session_rsrc,
							  &svc_name, &svc_name_len
							  ) == FAILURE) {
		return;
	}

	ZEND_FETCH_RESOURCE(zc_session, zeroconf_session *, &session_rsrc, -1, "ZeroConf Session", le_zeroconf);

	if (sw_discovery_browse(*(zc_session->session), svc_name, NULL, NULL, zc_browser_handler, zc_session, &id) != SW_OKAY) {
		RETURN_FALSE;
	}

	RETURN_TRUE;
}
/* }}} */

/* {{{ proto void zeroconf_browse_process(resource session, int timeout_msec)
   Process ZeroConf browsing for specified time interval.  */
PHP_FUNCTION(zeroconf_browse_process)
{
	zval *session_rsrc;
	zeroconf_session *zc_session;
	sw_salt salt;
	sw_ulong msecs;
	double timeout_secs;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rd",
							  &session_rsrc, &timeout_secs) == FAILURE) {
		return;
	}

	ZEND_FETCH_RESOURCE(zc_session, zeroconf_session *, &session_rsrc, -1, "ZeroConf Session", le_zeroconf);

	if (sw_discovery_salt(*(zc_session->session), &salt) != SW_OKAY) {
		fprintf(stderr, "salt failed\n");
		RETURN_FALSE;
	}

	msecs = (sw_ulong)timeout_secs * 1000;

	sw_salt_step(salt, &msecs);

	RETURN_TRUE;
}
/* }}} */

/* {{{ proto void zeroconf_session_destroy(resource session)
   End a ZeroConf session. */
PHP_FUNCTION(zeroconf_session_destroy)
{
	zval *session_rsrc;
	zeroconf_session *zc_session;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &session_rsrc) == FAILURE) {
		return;
	}

	ZEND_FETCH_RESOURCE(zc_session, zeroconf_session *, &session_rsrc, -1, "ZeroConf Session", le_zeroconf);

	zend_list_delete(Z_RESVAL_P(session_rsrc));
}
/* }}} */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
